@@ -229,7 +229,7 @@ recheck:
 
   if (!fw->have_bootloader_version
       && state == MXT_WAITING_BOOTLOAD_CMD) {
-    if (bootloader_id | 0x20) {
+    if (bootloader_id & 0x20) {
       mxt_dbg(fw->ctx, "Bootloader using extended ID mode");
       fw->extended_id_mode = true;
     } else {
@@ -362,6 +362,7 @@ static int send_frames(struct flash_context *fw)
       return ret;
     } else {
       mxt_verb(fw->ctx, "CRC pass");
+      frame_retry = 0;
       frame++;
       bytes_sent += frame_size;
       cur_percent = (unsigned char)(0.5f + (100.0 * ftell(fw->fp)) / fw->file_size);
